package com.iflytek.voicecloud.webapi.demo;

import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.iflytek.voicecloud.webapi.demo.util.HttpUtil;

/**
 * Machine Translation
 * @author iflytek
 */

public class WebITS {
    private static final String WebITS_URL = "https://its-api-sg.xf-yun.com/v2/its";
	// Access from the Console to find Machine Translation
	private static final String APPID = "XXXXXXXX";
	// Access from the Console to find Machine Translation
	private static final String API_KEY = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
	// Access from the Console to find Machine Translation
	private static final String API_SECRET = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
	// Source language
	private static final String FROM = "cn";
	// Target language
	private static final String TO = "ja";
	// Text data
	private static final String TEXT = "中华人民共和国于1949年成立";

	public static void main(String[] args) throws Exception {
		String body = buildHttpBody();
		//System.out.println("【ITSWebAPI body】\n" + body);
		Map<String, String> header = buildHttpHeader(body);
		Map<String, Object> resultMap = HttpUtil.doPost2(WebITS_URL, header, body);
		if (resultMap != null) {
			String resultStr = resultMap.get("body").toString();
			System.out.println("【its result】\n" + resultStr);

		    Gson json = new Gson();
	        ResponseData resultData = json.fromJson(resultStr, ResponseData.class);
	        int code = resultData.getCode();
	        if (resultData.getCode() != 0) {
	    		System.out.println("Please follow the error message tips to handle");
	        }
		} else {
			System.out.println("Please follow the error message tips to handle");
		}     
	}	

   public static Map<String, String> buildHttpHeader(String body) throws Exception {
		Map<String, String> header = new HashMap<String, String>();	
        URL url = new URL(WebITS_URL);
        
        //Generating a timestamp in RFC1123 format
        SimpleDateFormat format = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", Locale.US);
        format.setTimeZone(TimeZone.getTimeZone("GMT"));        
        Date dateD = new Date();
        String date = format.format(dateD);
		//System.out.println("【ITS WebAPI date】\n" + date);

		String digestBase64 = "SHA-256=" + signBody(body);
		//System.out.println("【ITS WebAPI digestBase64】\n" + digestBase64);
        
        StringBuilder builder = new StringBuilder("host: ").append(url.getHost()).append("\n").//
        		append("date: ").append(date).append("\n").//
                append("POST ").append(url.getPath()).append(" HTTP/1.1").append("\n").//
                append("digest: ").append(digestBase64);
		//System.out.println("【ITS WebAPI builder】\n" + builder);
        String sha = hmacsign(builder.toString(), API_SECRET);
		//System.out.println("【ITS WebAPI sha】\n" + sha);
		
        String authorization = String.format("api_key=\"%s\", algorithm=\"%s\", headers=\"%s\", signature=\"%s\"", API_KEY, "hmac-sha256", "host date request-line digest", sha);
        System.out.println("【ITS WebAPI authorization】\n" + authorization);
		
        header.put("Authorization", authorization);
        header.put("Content-Type", "application/json");
        header.put("Accept", "application/json,version=1.0");
        header.put("Host", url.getHost());
        header.put("Date", date);
        header.put("Digest", digestBase64);
		System.out.println("【ITS WebAPI header】\n" + header);
		return header;
    }
   
   public static String buildHttpBody() throws Exception {
       JsonObject body = new JsonObject();
       JsonObject business = new JsonObject();
       JsonObject common = new JsonObject();
       JsonObject data = new JsonObject();
       //common
       common.addProperty("app_id", APPID);
       //business
       business.addProperty("from", FROM);
       business.addProperty("to", TO);
       //data
       byte[] textByte = TEXT.getBytes("UTF-8");
       String textBase64 = new String(Base64.getEncoder().encodeToString(textByte));
       data.addProperty("text", textBase64);
       //body
       body.add("common", common);
       body.add("business", business);
       body.add("data", data);
       return body.toString();
   }

	private static String signBody(String body) throws Exception {
		MessageDigest messageDigest;
		String encodestr = "";
		try {
			messageDigest = MessageDigest.getInstance("SHA-256");
			messageDigest.update(body.getBytes("UTF-8"));
			encodestr = Base64.getEncoder().encodeToString(messageDigest.digest());
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		return encodestr;
	}
	private static String hmacsign(String signature, String apiSecret) throws Exception {
		Charset charset = Charset.forName("UTF-8");
		Mac mac = Mac.getInstance("hmacsha256");
		SecretKeySpec spec = new SecretKeySpec(apiSecret.getBytes(charset), "hmacsha256");
		mac.init(spec);
		byte[] hexDigits = mac.doFinal(signature.getBytes(charset));
		return Base64.getEncoder().encodeToString(hexDigits);
	}
	  
   public static class ResponseData {
       private int code;
       private String message;
       private String sid;
       private Object data;
       public int getCode() {
           return code;
       }
       public String getMessage() {
           return this.message;
       }
       public String getSid() {
           return sid;
       }
       public Object getData() {
           return data;
       }
   }
}
